package internal

import (
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/go-playground/validator/v10"
	"html"
	"net/http"
	"plugins/alipay_redpack/internal/po"
	"plugins/alipay_redpack/internal/vo"
	"plugins/utils/alipay"
	"plugins/utils/helper"
	"time"
)

type Config struct {
	AppId string `validate:"required" json:"app_id"`
	Prk   string `validate:"required" json:"prk"`
}

func (c *Config) validate() error {
	err := validator.New().Struct(c)
	if err != nil {
		for _, err = range err.(validator.ValidationErrors) {
			return proto.ErrorConfigFail(fmt.Sprintf("配置参数有误:%s", err.Error()))
		}
	}
	return nil
}

func transConfig(config []byte) (*Config, error) {
	var c Config
	err := json.Unmarshal(config, &c)
	if err != nil {
		return nil, proto.ErrorConfigFail(fmt.Sprintf("配置参数解析失败: %v", err))
	}
	if err = c.validate(); err != nil {
		return nil, err
	}
	return &c, nil
}

func (c *Config) paramReq(req po.Req, method string) (*po.Param, error) {
	if err := req.Validate(); err != nil {
		return nil, err
	}
	cert, err := alipay.GetCert(c.AppId)
	if err != nil {
		return nil, proto.ErrorConfigFail(err.Error())
	}
	return &po.Param{
		AlipayRootCertSn: cert.RootCertSN,
		AppCertSn:        cert.MchCertSN,
		AppId:            c.AppId,
		Method:           method,
		Format:           vo.Format,
		Charset:          vo.Charset,
		SignType:         vo.SignType,
		Timestamp:        time.Now().Format(time.DateTime),
		Version:          vo.Version,
		BizContent:       string(req.ToJson()),
		Sign:             "",
	}, nil
}

func orderReq(order *proto.OrderRequest_Order, product *proto.OrderRequest_Product) (*po.OrderReq, error) {
	type ProductExtra struct {
		Wishing string `json:"wishing"`
	}
	var productExtra ProductExtra
	if product.Extra == nil {
		return nil, fmt.Errorf("商品拓展参数不能为空")
	}
	err := json.Unmarshal(product.Extra, &productExtra)
	if err != nil {
		return nil, fmt.Errorf("商品拓展参数 json unmarshal error: %v", err)
	}
	payeeInfo := po.PayeeInfo{
		Identity:     order.Account,
		IdentityType: "ALIPAY_USER_ID",
	}
	if helper.IsPhoneNumber(order.Account) || helper.IsEmail(order.Account) {
		if order.Extra == nil {
			return nil, fmt.Errorf("order extra is nil")
		}
		type OrderExtra struct {
			Name string `json:"name"`
		}
		var orderExtra OrderExtra
		err = json.Unmarshal(order.Extra, &orderExtra)
		if err != nil {
			return nil, fmt.Errorf("订单拓展参数 json unmarshal error: %v", err)
		}
		if orderExtra.Name == "" {
			return nil, fmt.Errorf("姓名，当identity_type=ALIPAY_LOGON_ID时，本字段必填")
		}
		payeeInfo.IdentityType = "ALIPAY_LOGON_ID"
		payeeInfo.Name = orderExtra.Name
	}
	return &po.OrderReq{
		OutBizNo:       order.OrderNo,
		TransAmount:    order.Amount,
		ProductCode:    "STD_RED_PACKET",
		BizScene:       "DIRECT_TRANSFER",
		OrderTitle:     productExtra.Wishing,
		Remark:         productExtra.Wishing,
		PayeeInfo:      &payeeInfo,
		BusinessParams: `{"sub_biz_scene":"REDPACKET"}`,
	}, nil
}

func orderResp(request *proto.OrderRequest, resp *po.OrderResp) *proto.OrderResponse {
	data, _ := json.Marshal(resp)
	return &proto.OrderResponse{
		Result: &proto.Result{
			Status:  resp.GetOrderStatus(),
			OrderNo: request.Order.OrderNo,
			TradeNo: resp.Response.OrderId,
			Message: resp.GetMsg(),
			Data:    data,
		},
	}
}

func queryReq(in *proto.QueryRequest_Order) *po.QueryReq {
	return &po.QueryReq{
		OutBizNo:    in.OrderNo,
		OrderId:     in.TradeNo,
		ProductCode: "STD_RED_PACKET",
		BizScene:    "DIRECT_TRANSFER",
	}
}

func queryResp(request *proto.QueryRequest, resp po.QueryResp) *proto.QueryResponse {
	data, _ := json.Marshal(resp)
	return &proto.QueryResponse{
		Result: &proto.Result{
			Status:  resp.Data.Status.GetOrderStatus(),
			OrderNo: request.Order.OrderNo,
			TradeNo: request.Order.TradeNo,
			Message: resp.Data.Msg,
			Data:    data,
		},
	}
}

func notifyReq(in *proto.NotifyRequest) *po.Notify {
	var n *po.Notify
	_ = json.Unmarshal(in.Queries, &n)
	n.BizContent = html.UnescapeString(n.BizContent)
	return n
}

func notifyResp(n *po.Notify) (*proto.NotifyResponse, error) {
	var b po.NotifyBizContent
	_ = json.Unmarshal([]byte(n.BizContent), &b)

	pb := &proto.NotifyResponse{
		Result: &proto.Result{
			Status:  b.Status.GetOrderStatus(),
			OrderNo: b.OutBizNo,
			TradeNo: b.OrderId,
			Message: b.Status.GetText(),
		},
		Return: "success",
	}

	headers := make(http.Header)
	headers.Set("Content-Type", "text/plain")

	headersBytes, err := json.Marshal(headers)
	if err != nil {
		return nil, err
	}
	pb.Headers = string(headersBytes)

	return pb, nil
}
