package internal

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"io"
	"net/http"
)

// 插件通信信息,若不对应则会报错panic
const (
	Tag         = "zltx_card_v1"
	Version     = 1
	CookieKey   = "zltx_card_v1"
	CookieValue = "zltx_card_v1"
)

type ZLTXCardV1Service struct{}

func (p *ZLTXCardV1Service) Order(ctx context.Context, request *proto.OrderRequest) (resp2 *proto.OrderResponse, respErr error) {

	select {
	case <-ctx.Done():
		return nil, ctx.Err()
	default:
		// 继续执行
	}

	defer func() {
		if err := recover(); err != nil {
			respErr = fmt.Errorf("panic: %v", err)
		}
	}()

	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, err := dctW.Order(ctx, c.orderReq(request))
	if err != nil {
		if proto.IsResponseFail(err) {
			return &proto.OrderResponse{Result: Result(err.Error())}, nil
		}
		return nil, err
	}
	if !resp.GetCode().IsSuccess() {
		return nil, proto.ErrorRequestFail(resp.Message)
	}

	return orderResp(request, resp), nil
}

func (p *ZLTXCardV1Service) Query(ctx context.Context, request *proto.QueryRequest) (*proto.QueryResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, cardCode, err := dctW.Query(ctx, c.queryReq(request))
	if err != nil {
		return nil, err
	}
	if !resp.GetCode().IsSuccess() {
		return nil, proto.ErrorRequestFail(resp.Message)
	}

	return queryResp(request, resp, cardCode)
}

func (p *ZLTXCardV1Service) Notify(ctx context.Context, request *proto.NotifyRequest) (resp2 *proto.NotifyResponse, respErr error) {

	select {
	case <-ctx.Done():
		return nil, ctx.Err()
	default:
		// 继续执行
	}

	defer func() {
		if err := recover(); err != nil {
			respErr = fmt.Errorf("panic: %v", err)
		}
	}()

	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	httpHeaders := make(http.Header)
	if err = json.Unmarshal(request.Headers, &httpHeaders); err != nil {
		return nil, proto.ErrorParamFail(fmt.Sprintf("headers Unmarshal err [%v]", err))
	}
	newHeaders := make(http.Header)
	for key, values := range httpHeaders {
		newHeaders[key] = values
	}

	httpRequest := &http.Request{
		Header: newHeaders,
		Body:   io.NopCloser(bytes.NewBuffer(request.Body)),
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, cardCode, err := dctW.Notify(ctx, httpRequest)
	if err != nil {
		return nil, err
	}

	return notifyResp(resp, cardCode)
}
